#ifndef __CSAFEARRAYUTILS_H__
#define __CSAFEARRAYUTILS_H__

// *****************************************************************************
// 
// *****************************************************************************

#include <comdef.h>
#include <string>
#include <atlbase.h>
#include <ExceptionUtils.h>
#include <TNT.h>

/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/

/// <summary>useful class manipulate safarray</summary>
class CSAFEARRAYUTILS{
protected:
	/// <summary>Put a data in a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static inline void saPutData(SAFEARRAY* rarray, long* index, void* data) throw(...);

	/// <summary>Get a data from a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static inline void saGetData(SAFEARRAY* rarray, long* index, void* data) throw(...);

	/// <summary>Get back actual type of safearray</summary>
	/// <param name="rarray">safearray</param>
	/// <returns>VARTYPE type of the safearray element</returns>
	static VARTYPE get_type(SAFEARRAY* rarray);

	/// <summary>Put a variant in a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saPut(SAFEARRAY* rarray, long* index, VARIANT& data) throw(...);

	/// <summary>Get a variant from a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saGet(SAFEARRAY* rarray, long* index, VARIANT* data) throw(...);

public:
	/// <summary>Get the dimensions of a safearray </summary>
	/// <param name="rarray">the safe array</param>
	/// <param name="nbelemPerDim">elements for each dimension</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saGetDim(SAFEARRAY *rarray, TNT::Array1D<long>& nbelemPerDim) throw(...);

	/// <summary>Create a safearray of dimension n</summary>
	/// <param name="vt">type of the safe array to create</param>
	/// <param name="count">array of size of the array</param>
	/// <param name="varOut">Variant to create</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saCreate(VARTYPE vt, long* count, int dim, VARIANT& varOut) throw(...);

	/// <summary>Put a string in a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saPut(SAFEARRAY *rarray, long* index, const char* data) throw(...);

	/// <summary>Put a wide char string in a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saPut(SAFEARRAY *rarray, long* index, const wchar_t* data) throw(...);

	/// <summary>Put a IDispatch interface in a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saPut(SAFEARRAY* rarray, long* index, IDispatch* data) throw(...);
	
	/// <summary>Put a wstring in a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saPut(SAFEARRAY* rarray, long* index, const std::wstring& data) throw(...);
	
	/// <summary>Put a double in a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saPut(SAFEARRAY* rarray, long* index, const double& data) throw(...);

	/// <summary>Put a long in a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saPut(SAFEARRAY* rarray, long* index, const long& data) throw(...);

	/// <summary>Get a string from a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saGet(SAFEARRAY *rarray, long* index, char** data) throw(...);

	/// <summary>Get a wide char string from a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saGet(SAFEARRAY *rarray, long* index, wchar_t** data) throw(...);

	/// <summary>Get a IDispatch interface from a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saGet(SAFEARRAY* rarray, long* index, IDispatch** data) throw(...);
	
	/// <summary>Get a wstring from a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saGet(SAFEARRAY* rarray, long* index, std::wstring* data) throw(...);
	
	/// <summary>Get a double from a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	/// <remarks>Works for long / double type of safearray</remarks>
	static void saGet(SAFEARRAY* rarray, long* index, double* data) throw(...);

	/// <summary>Get a long from a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saGet(SAFEARRAY* rarray, long* index, long* data) throw(...);

	/// <summary>Get a bool from a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saGet(SAFEARRAY* rarray, long* index, bool* data) throw(...);

	/// <summary>Put a bool in a safearray at a given location</summary>
	/// <param name="array">safearray</param>
	/// <param name="index">location</param>
	/// <param name="data">data to store in the safearray</param>
	/// <exception cref="R2PException::CMyException">Thrown on error</exception>
	/// <returns>void</returns>
	static void saPut(SAFEARRAY* rarray, long* index, const bool& data) throw(...);
};
/****************************************************************************************************************************
*	inline method
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saPutData(SAFEARRAY *rarray, long* index, void* data){
	HRESULT hr = E_FAIL;
	try{
		hr = SafeArrayPutElement(rarray, index, data);
		R2PException::Utils::ThrowException(SUCCEEDED(hr), hr, L"Array Error", L"Error while inserting element in array.\n");
	}
	catch(R2PException::CMyException& herr){
		throw herr;
	}
	catch(...){
		throw R2PException::CMyException(E_UNEXPECTED, L"Array Error",  L"Error while inserting element in array.\n");
	}
}
/****************************************************************************************************************************
*	inline method
*
****************************************************************************************************************************/
void CSAFEARRAYUTILS::saGetData(SAFEARRAY* rarray, long* index, void* data){
	HRESULT hr = E_FAIL;
	try{
		hr = SafeArrayGetElement(rarray, index, data);
		R2PException::Utils::ThrowException(SUCCEEDED(hr), hr, L"Array Error", L"Error while getting element from array.\n");
	}
	catch(R2PException::CMyException& herr){
		throw herr;
	}
	catch(...){
		throw R2PException::CMyException(E_UNEXPECTED, L"Array Error", L"Error while getting element from array.\n");
	}
}


#endif  //  __CSAFEARRAYUTILS_H__
