#include "ExceptionUtils.h"

#include "CCapeBadArgument.h"
#include "CCapeBadCOParameter.h"
#include "CCapeBadInvOrder.h"
#include "CCapeBoundaries.h"
#include "CCapeComputation.h"
#include "CCapeData.h"
#include "CCapeFailedInitialisation.h"
#include "CCapeIllegalAccess.h"
#include "CCapeImplementation.h"
#include "CCapeInvalidArgument.h"
#include "CCapeInvalidOperation.h"
#include "CCapeLicenceError.h"
#include "CCapeLimitedImpl.h"
#include "CCapeNoImpl.h"
#include "CCapeNoMemory.h"
#include "CCapeOutOfBounds.h"
#include "CCapeOutOfResources.h"
#include "CCapePersistence.h"
#include "CCapePersistenceNotFound.h"
#include "CCapePersistenceOverflow.h"
#include "CCapePersistenceSystemError.h"
#include "CCapeSolvingError.h"
#include "CCapeTimeOut.h"
#include "CCapeUnknown.h"
#include "CCapeUser.h"

namespace R2PException{
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
	eCapeErrorInterfaceHR_tag Utils::HRES2CapeError(HRESULT hres, CCapeUser** pErr){
		eCapeErrorInterfaceHR_tag capeCode;
		switch(hres){
			case E_NOTIMPL:
				*pErr = new CCapeNoImpl(hres, L"Operation not implemented.\n");
				capeCode = ECapeNoImplHR;
				break;
			case E_OUTOFMEMORY:
				*pErr = new CCapeNoMemory(hres, L"No more memory.\n");
				capeCode = ECapeNoMemoryHR;
				break;
			case E_UNEXPECTED:
				*pErr = new CCapeUnknown(hres, L"Unexpected exception.\n");
				capeCode = ECapeUnknownHR;
				break;
			case REGDB_E_READREGDB:
				*pErr = new CCapeData(hres, L"Bad HKEY.\n");
				capeCode = ECapeDataHR;
				break;
			case DISP_E_BADINDEX:
				*pErr = new CCapeData(hres, L"Bad index.\n");
				capeCode = ECapeDataHR;
				break;
			case E_INVALIDARG:
				*pErr = new CCapeData(hres, L"Bad argument.\n");
				capeCode = ECapeDataHR;
				break;
			case REGDB_E_CLASSNOTREG:
				*pErr = new CCapeFailedInitialisation(hres, L"Interface is not registered in the registration database or type of server is not registered or the values for the server types in the registry are corrupt. Try to reinstall software.\n");
				capeCode = ECapeFailedInitialisationHR;
				break;
			case CLASS_E_NOAGGREGATION:
				*pErr = new CCapeFailedInitialisation(hres, L"Interface cannot be created as part of an aggregate.\n");
				capeCode = ECapeFailedInitialisationHR;
				break;
			case E_NOINTERFACE:
				*pErr = new CCapeFailedInitialisation(hres, L"The interface does not implement the requested interface.\n");
				capeCode = ECapeFailedInitialisationHR;
				break;
			case CO_E_CLASSSTRING:
				*pErr = new CCapeFailedInitialisation(hres, L"Windows registry is corrupt. Please, reinstall packages.\n");
				capeCode = ECapeFailedInitialisationHR;
				break;
			case DISP_E_ARRAYISLOCKED:
				*pErr = new CCapeComputation(hres, L"One of the arguments contains an array that is locked.\n");
				capeCode = ECapeComputationHR;
				break;
			case DISP_E_BADVARTYPE:
				*pErr = new CCapeComputation(hres, L"The source and destination argument have an invalid type (usually uninitialized).\n");
				capeCode = ECapeComputationHR;
				break;
			case R2P_EMATHS:
				*pErr = new CCapeSolvingError(hres);
				capeCode = ECapeSolvingErrorHR;
				break;
			case R2P_EBADORDER:
				*pErr = new CCapeBadInvOrder(hres);
				capeCode = ECapeBadInvOrderHR;
				break;
			case R2P_EINVARG:
				*pErr = new CCapeInvalidArgument(hres);
				capeCode = ECapeInvalidArgumentHR;
				break;
			case R2P_EOUTOFBOUNDS:
				*pErr = new CCapeOutOfBounds(hres);
				capeCode = ECapeOutOfBoundsHR;
				break;
			case R2P_EBADARG:
				*pErr = new CCapeBadArgument(hres);
				capeCode = ECapeBadArgumentHR;
				break;
			case R2P_EFAILEDINIT:
				*pErr = new CCapeFailedInitialisation(hres);
				capeCode = ECapeFailedInitialisationHR;
				break;
			case R2P_ENOIMPL:
				*pErr = new CCapeNoImpl(hres);
				capeCode = ECapeNoImplHR;
				break;
			case R2P_ELICENCE:
				*pErr = new CCapeLicenceError(hres);
				capeCode = ECapeLicenceErrorHR;
				break;
			case R2P_ECOMPUTATION:
				*pErr = new CCapeComputation(hres);
				capeCode = ECapeComputationHR;
				break;
			case R2P_SWARN:
				*pErr = new CCapeUnknown(hres);
				break;
			default:
				std::wstring errmess = L"Failed to interpret error code ";
				wchar_t tmp[100];
				_ltow_s(hres, tmp, 100, 10);
				errmess += tmp;
				errmess += L"\n";
				*pErr = new CCapeUnknown(hres, errmess);
				capeCode = ECapeUnknownHR;
		}
		return capeCode;
	}
/****************************************************************************************************************************
*								
*
****************************************************************************************************************************/
	CCapeUser* Utils::OutScope2InScope(HRESULT hres, const IDispatchPtr& pDisp){
		CCapeUser* pVal = NULL;
		switch(hres){
			case ECapeBadArgumentHR:
				pVal = new CCapeBadArgument(pDisp);
				break;
			case ECapeBadCOParameterHR:
				pVal = new CCapeBadCOParameter(pDisp);
				break;
			case ECapeBadInvOrderHR:
				pVal = new CCapeBadInvOrder(pDisp);
				break;
			case ECapeComputationHR:
				pVal = new CCapeComputation(pDisp);
				break;
			case ECapeDataHR:
				pVal = new CCapeData(pDisp);
				break;
			case ECapeFailedInitialisationHR:
				pVal = new CCapeFailedInitialisation(pDisp);
				break;
			case ECapeIllegalAccessHR:
				pVal = new CCapeIllegalAccess(pDisp);
				break;
			case ECapeImplementationHR:
				pVal = new CCapeImplementation(pDisp);
				break;
			case ECapeInvalidArgumentHR:
				pVal = new CCapeInvalidArgument(pDisp);
				break;
			case ECapeInvalidOperationHR:
				pVal = new CCapeInvalidOperation(pDisp);
				break;
			case ECapeLicenceErrorHR:
				pVal = new CCapeLicenceError(pDisp);
				break;
			case ECapeLimitedImplHR:
				pVal = new CCapeLimitedImpl(pDisp);
				break;
			case ECapeNoImplHR:
				pVal = new CCapeNoImpl(pDisp);
				break;
			case ECapeNoMemoryHR:
				pVal = new CCapeNoMemory(pDisp);
				break;
			case ECapeOutOfBoundsHR:
				pVal = new CCapeOutOfBounds(pDisp);
				break;
			case ECapeOutOfResourcesHR:
				pVal = new CCapeOutOfResources(pDisp);
				break;
			case ECapePersistenceHR:
				pVal = new CCapePersistence(pDisp);
				break;
			case ECapePersistenceNotFoundHR:
				pVal = new CCapePersistenceNotFound(pDisp);
				break;
			case ECapePersistenceOverflowHR:
				pVal = new CCapePersistenceOverflow(pDisp);
				break;
			case ECapePersistenceSystemErrorHR:
				pVal = new CCapePersistenceSystemError(pDisp);
				break;
			case ECapeSolvingErrorHR:
				pVal = new CCapeSolvingError(pDisp);
				break;
			case ECapeTimeOutHR:
				pVal = new CCapeTimeOut(pDisp);
				break;
			case ECapeUnknownHR:
				pVal = new CCapeUnknown(pDisp);
				break;
			default:
				pVal = new CCapeUnknown(pDisp);
				break;
		}
		return pVal;
	}

}