#if !defined(__EXCEPTION_UTILS_H__)
#define __EXCEPTION_UTILS_H__

#include <atlbase.h>
#include <iostream>
#include "CCapeUser.h"
#include "CCapeUnknown.h"
#include "CMyException.h"

namespace R2PException{
	enum SpecificHRES{
		R2P_EMATHS = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, 5),
		R2P_EBADORDER = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, 6),
		R2P_EINVARG = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, 7),
		R2P_EOUTOFBOUNDS = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, 8),
		R2P_EBADARG = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, 9),
		R2P_EFAILEDINIT = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, 10),
		R2P_ENOIMPL = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, 11),
		R2P_ELICENCE = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, 12),
		R2P_ECOMPUTATION = MAKE_HRESULT(SEVERITY_ERROR, FACILITY_ITF, 13),
		R2P_SWARN = MAKE_HRESULT(SEVERITY_SUCCESS, FACILITY_ITF, 20)
	};
	/// <summary>Utility class to manage CAPE-OPEN exception</summary>
	class Utils{
	public:

		/// <summary>inline method to check if exception has to be thrown</summary>
		/// <param name="cond">condition to check: if true no exception, otherwise exception is thrown</param>
		/// <param name="errcode">exception error code</param>
		/// <param name="name">short description of the exception</param>
		/// <param name="format">more detailed description following printf format</param>
		/// <param name="...">va_list that describes dynamic parameter from format argument</param>
		/// <returns>void</returns>
		/// <exception cref="R2PException::CMyException">Thrown on false condition</exception>
		static inline void ThrowException(bool cond, HRESULT errcode, const wchar_t* name, const wchar_t* format, ...) throw(...){
			if(!(cond)){
				//Rebuild the va_list in order to pass it to wsprintf
				va_list marker;
				va_start( marker, format );

				size_t nbElem = _vscwprintf( format, marker ) + 1;

				wchar_t* buffer = (wchar_t*)alloca(sizeof(wchar_t) * nbElem);
				memset(buffer, 0, sizeof(wchar_t) * nbElem);
				vswprintf_s( buffer, /*_countof( buffer )*/nbElem, format, marker );

				va_end( marker );
				throw R2PException::CMyException(errcode, name, buffer);
			}
		}

		/// <summary>inline method to check if exception has to be thrown</summary>
		/// <param name="cond">condition to check: if true no exception, otherwise exception is thrown</param>
		/// <param name="herr">exception objects</param>
		/// <returns>void</returns>
		/// <exception cref="R2PException::CMyException">Thrown on false condition</exception>
		static inline void ThrowException(bool cond, CMyException* herr) throw(...){
			if(!(cond))
				throw *herr;
		}

		/// <summary>inline method to check if warning has to be generated</summary>
		/// <param name="cond">condition to check: if true no warning, otherwise warning is generated</param>
		/// <param name="pWarn">pointer to warning handler (it is a R2PException::CMyException object)</param>
		/// <param name="name">short description of the warning</param>
		/// <param name="format">more detailed description following printf format</param>
		/// <param name="...">va_list that describes dynamic parameter from format argument</param>
		/// <returns>void</returns>
		/// <exception cref="R2PException::CMyException">Thrown on false condition</exception>
		static inline HRESULT ThrowWarning(bool cond, R2PException::CMyException** pWarn, const wchar_t* name, const wchar_t* format, ...) throw(){
			HRESULT hr = S_OK;
			if(!(cond)){
				//Rebuild the va_list in order to pass it to wsprintf
				va_list marker;
				va_start( marker, format );

#if 0
				wchar_t buffer[1024] = { 0 };
				vswprintf_s( buffer, _countof( buffer ), format, marker );
#else
				size_t nbElem = _vscwprintf( format, marker ) + 1;

				wchar_t* buffer = (wchar_t*)alloca(sizeof(wchar_t) * nbElem);
				memset(buffer, 0, sizeof(wchar_t) * nbElem);
				vswprintf_s( buffer, /*_countof( buffer )*/nbElem, format, marker );
#endif

				va_end( marker );
				if(*pWarn) delete *pWarn;
				*pWarn = new R2PException::CMyException(R2PException::R2P_SWARN, name, buffer);
				hr = R2PException::R2P_SWARN;
			}
			return hr;
		}

		/// <summary>inline method to check if exception has to be thrown</summary>
		/// <param name="cond">condition to check: if true no exception, otherwise exception is thrown</param>
		/// <param name="errcode">exception error code</param>
		/// <param name="pDisp">CAPE-OPEN interface that throws exception</param>
		/// <returns>void</returns>
		/// <exception cref="R2PException::CMyException">Thrown on false condition</exception>
		static inline void ThrowException(bool cond, HRESULT errcode, IDispatch* pDisp) throw(...){
			if(!cond)
				throw R2PException::CMyException(errcode, pDisp);
		}

		/// <summary>Create error from an HRESULT error code</summary>
		/// <param name="hres">the HRESULT code that define error</param>
		/// <param name="pErr">error to generate</param>
		/// <returns>CAPE-OPEN error code</returns>
		static eCapeErrorInterfaceHR_tag HRES2CapeError(HRESULT hres, CCapeUser** pErr);

		/// <summary>Convert a CAPE-OPEN error (wrapped into IDispatch interface) into a class defined object</summary>
		/// <param name="hres">the HRESULT code that define error</param>
		/// <param name="pDisp">CAPE-OPEN interface that throws exception</param>
		/// <param name="CompName">CAPE-OPEN interface name that throws exception</param>
		/// <returns>CCapeUser pointer to the CAPE-OPEN error object</returns>
		static CCapeUser* OutScope2InScope(HRESULT hres, const IDispatchPtr& pDisp);
	};
}

#endif //__EXCEPTION_UTILS_H__